@@ -6,6 +6,9 @@
  *  * kqueue() implementation
  *  * windows port (yeah, right, volunteers are DEFINITELY needed for this one...)
  */
+#if HAVE_PROTOBUF_C_CONFIG_H
+#include "protobuf-c-config.h"
+#endif
 #include <assert.h>
 #if HAVE_ALLOCA_H
 # include <alloca.h>
@@ -417,7 +420,7 @@ deallocate_notify_desired_index (RealDispatch *d,
 /* Registering file-descriptors to watch. */
 void
 protobuf_c_dispatch_watch_fd (ProtobufCDispatch *dispatch,
-                              int                 fd,
+                              ProtobufC_FD        fd,
                               unsigned            events,
                               ProtobufCDispatchCallback callback,
                               void               *callback_data)
@@ -487,7 +490,7 @@ protobuf_c_dispatch_watch_fd (ProtobufCDispatch *dispatch,
 
 void
 protobuf_c_dispatch_close_fd (ProtobufCDispatch *dispatch,
-                              int                 fd)
+                              ProtobufC_FD        fd)
 {
   protobuf_c_dispatch_fd_closed (dispatch, fd);
   close (fd);
@@ -567,6 +570,7 @@ protobuf_c_dispatch_dispatch (ProtobufCDispatch *dispatch,
       idle->next = d->recycled_idles;
       d->recycled_idles = idle;
     }
+  dispatch->has_idle = 0;
 
   /* handle timers */
   gettimeofday (&tv, NULL);
@@ -654,26 +658,22 @@ protobuf_c_dispatch_run (ProtobufCDispatch *dispatch)
     }
 
   /* compute timeout */
-  if (d->first_idle != NULL)
-    {
-      timeout = 0;
-    }
-  else if (d->timer_tree == NULL)
+  if (dispatch->has_idle)
+    timeout = 0;
+  else if (!dispatch->has_timeout)
     timeout = -1;
   else
     {
-      ProtobufCDispatchTimer *min_timer;
-      GSK_RBTREE_FIRST (GET_TIMER_TREE (d), min_timer);
       struct timeval tv;
       gettimeofday (&tv, NULL);
-      if (min_timer->timeout_secs < (unsigned long) tv.tv_sec
-       || (min_timer->timeout_secs == (unsigned long) tv.tv_sec
-        && min_timer->timeout_usecs <= (unsigned) tv.tv_usec))
+      if (dispatch->timeout_secs < (unsigned long) tv.tv_sec
+       || (dispatch->timeout_secs == (unsigned long) tv.tv_sec
+        && dispatch->timeout_usecs <= (unsigned) tv.tv_usec))
         timeout = 0;
       else
         {
-          int du = min_timer->timeout_usecs - tv.tv_usec;
-          int ds = min_timer->timeout_secs - tv.tv_sec;
+          int du = dispatch->timeout_usecs - tv.tv_usec;
+          int ds = dispatch->timeout_secs - tv.tv_sec;
           if (du < 0)
             {
               du += 1000000;
@@ -833,6 +833,7 @@ protobuf_c_dispatch_add_idle (ProtobufCDispatch *dispatch,
   rv->func = func;
   rv->func_data = func_data;
   rv->dispatch = d;
+  dispatch->has_idle = 1;
   return rv;
 }
 
