@@ -66,18 +66,19 @@ void PrimitiveFieldGenerator::GenerateStructMembers(io::Printer* printer) const
   }
   vars["c_type"] = c_type;
   vars["name"] = FieldName(descriptor_);
+  vars["deprecated"] = FieldDeprecated(descriptor_);
 
   switch (descriptor_->label()) {
     case FieldDescriptor::LABEL_REQUIRED:
-      printer->Print(vars, "$c_type$ $name$;\n");
+      printer->Print(vars, "$c_type$ $name$$deprecated$;\n");
       break;
     case FieldDescriptor::LABEL_OPTIONAL:
-      printer->Print(vars, "protobuf_c_boolean has_$name$;\n");
-      printer->Print(vars, "$c_type$ $name$;\n");
+      printer->Print(vars, "protobuf_c_boolean has_$name$$deprecated$;\n");
+      printer->Print(vars, "$c_type$ $name$$deprecated$;\n");
       break;
     case FieldDescriptor::LABEL_REPEATED:
-      printer->Print(vars, "size_t n_$name$;\n");
-      printer->Print(vars, "$c_type$ *$name$;\n");
+      printer->Print(vars, "size_t n_$name$$deprecated$;\n");
+      printer->Print(vars, "$c_type$ *$name$$deprecated$;\n");
       break;
   }
 }
